<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace AlizeLib;

    use AlizeLib\Objects\Label;

    class GeneralizationModel
    {
        /**
         * @var Label[]
         */
        private $labels;

        /**
         * @var int
         */
        private $size;

        public function __construct(int $size)
        {
            $this->labels = [];
            $this->size = $size;
        }

        /**
         * Returns the label with the given name
         *
         * @param string $label
         * @return Label|null
         */
        public function getLabel(string $label): ?Label
        {
            return $this->labels[$label] ?? null;
        }

        /**
         * @return int
         */
        public function getSize(): int
        {
            return $this->size;
        }

        /**
         * Resizes the model to the given size
         *
         * @param int $size
         * @return void
         */
        public function resize(int $size): void
        {
            foreach($this->labels as $label)
            {
                $label->resize($size);
            }
        }

        /**
         * Adds a value to the label with the given name
         *
         * @param string $label
         * @param $value
         * @return void
         */
        public function add(string $label, $value): void
        {
            if(!isset($this->labels[$label]))
            {
                $this->labels[$label] = new Label($label, $this->size);
            }

            $this->labels[$label]->addValue($value);
        }

        /**
         * Returns the data array for the label with the given name
         *
         * @param string $label
         * @return array
         */
        public function getData(string $label): array
        {
            if(!isset($this->labels[$label]))
            {
                return [];
            }

            return $this->labels[$label]->getData();
        }

        /**
         * Calculates the mean for each label
         *
         * @return array
         */
        public function calculateMean(): array
        {
            $means = [];
            /** @var Label $label */
            foreach($this->labels as $label)
            {
                $means[$label->getLabel()] = $label->calculateMean();
            }

            return $means;
        }

        /**
         * Calculates the standard deviation for each label
         *
         * @return array
         */
        public function calculateStandardDeviation(): array
        {
            $stds = [];
            /** @var Label $label */
            foreach($this->labels as $label)
            {
                $stds[$label->getLabel()] = $label->calculateConfidence();
            }

            return $stds;
        }

        /**
         * Calculates the best labels based on the mean and standard deviation
         * The best labels are the labels with the lowest mean + standard deviation
         *
         * @return array
         */
        public function calculateBestLabels(): array
        {
            $means = $this->calculateMean();
            $stds = $this->calculateStandardDeviation();
            $results = [];

            foreach($this->labels as $name => $label)
            {
                $filtered_data = array_filter($label->getData(), function($value) use ($means, $stds, $name) {
                    return $value >= $means[$name] - $stds[$name];
                });

                if(count($filtered_data) === 0)
                {
                    $results[$name] = 0;
                    continue;
                }

                $results[$name] = array_sum($filtered_data) / count($filtered_data);
            }

            arsort($results);
            return $results;
        }

    }