<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace AlizeLib\Objects;

    class Label
    {
        /**
         * The name of the label
         *
         * @var string
         */
        private $label;

        /**
         * An array of the data for this label
         *
         * @var float[]|int[]|double[]
         */
        private $data;

        /**
         * The current pointer for the data array
         *
         * @var int
         */
        private $pointer;

        /**
         * @var int
         */
        private $size;

        /**
         * @param string $label
         * @param int $size
         */
        public function __construct(string $label, int $size)
        {
            $this->label = $label;
            $this->data = [];
            $this->size = $size;
            $this->pointer = 0;
        }

        /**
         * Returns the name of the label
         *
         * @return string
         */
        public function getLabel(): string
        {
            return $this->label;
        }

        /**
         * Returns the data array for this label
         *
         * @return array
         */
        public function getData(): array
        {
            return $this->data;
        }

        /**
         * Returns the current size set for the data array (not the actual size of the array)
         *
         * @return int
         */
        public function getSize(): int
        {
            return $this->size;
        }

        /**
         * Resizes the data array to the given size
         *
         * @param int $size
         * @return void
         */
        public function resize(int $size): void
        {
            $this->size = $size;
            if($this->pointer > $size)
                $this->pointer = $size;
            if(count($this->data) > $size)
                $this->data = array_slice($this->data, 0, $size);
        }

        /**
         * Adds a value to the data array
         *
         * @param $value
         * @return void
         */
        public function addValue($value): void
        {
            if($this->pointer > $this->size)
                $this->pointer = 0;

            $this->data[$this->pointer] = $value;
            $this->pointer += 1;
        }

        /**
         * Adds an array of values to the data array
         *
         * @param array $values
         * @return void
         */
        public function addValues(array $values): void
        {
            foreach($values as $value)
                $this->addValue($value);
        }

        /**
         * Resets the data array and pointer to empty and 0 respectively
         *
         * @return void
         */
        public function clear(): void
        {
            $this->data = [];
            $this->pointer = 0;
        }

        /**
         * Calculates the mean of the data array by adding all the values and dividing by the number of values
         *
         * @return float|int
         */
        public function calculateMean(): float|int
        {
            if(count($this->data) === 0)
                return 0;

            return array_sum($this->data) / count($this->data);
        }

        /**
         * Calculates the variance of the data array by subtracting the mean from each value, squaring the result, adding
         * all the results together and dividing by the number of values in the array (n) to get the variance of the
         * data set.
         *
         * @return float|int
         */
        public function calculateConfidence(): float|int
        {
            if(count($this->data) === 0)
                return 0;

            $mean = $this->calculateMean();
            $variance = 0;
            foreach($this->data as $value)
            {
                $variance += pow($value - $mean, 2);
            }
            return $variance / count($this->data);
        }

        /**
         * Calculates the standard deviation of the data array by taking the square root of the variance
         *
         * @return float|int
         */
        public function total(): float|int
        {
            return array_sum($this->data);
        }

        /**
         * Returns an array representation of the label
         *
         * @return array
         */
        public function toArray(): array
        {
            return [
                'label' => $this->label,
                'data' => $this->data,
                'size' => $this->size,
                'pointer' => $this->pointer
            ];
        }

        /**
         * Constructs a label from an array representation
         *
         * @param array $array
         * @return Label
         */
        public static function fromArray(array $array): Label
        {
            $label = new Label($array['label'], $array['size']);
            $label->data = $array['data'];
            $label->pointer = $array['pointer'];
            return $label;
        }
    }